/*
 * @(#) $Id:  $
 *
 * Copyright 2009/2010 by sIT Solutions,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package org.openxma.dsl.core.linking;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.TerminalRule;
import org.eclipse.xtext.diagnostics.IDiagnosticConsumer;
import org.eclipse.xtext.linking.lazy.LazyLinker;
import org.eclipse.xtext.nodemodel.ICompositeNode;
import org.eclipse.xtext.nodemodel.INode;
import org.eclipse.xtext.nodemodel.util.NodeModelUtils;
import org.openxma.dsl.core.linking.support.JavaDocStringBuilder;
import org.openxma.dsl.core.model.IDocumentable;

/**
 * This linker parses JavaDoc comments between elements and adds them to related model elements
 * @author s6526  
 * @since version_number
 */
public class DocumentationLinker extends LazyLinker {
    
    @Override
    protected void afterModelLinked(EObject model, IDiagnosticConsumer diagnosticsConsumer) {
        ICompositeNode compositeNode = NodeModelUtils.getNode(model);
        ICompositeNode rootNode = compositeNode.getRootNode();
        JavaDocStringBuilder javaDocStringBuilder = new JavaDocStringBuilder();
        for(INode node : rootNode.getAsTreeIterable()) {
            EObject grammarElement = node.getGrammarElement();
            if (grammarElement instanceof TerminalRule) {
                javaDocStringBuilder.newText(node.getText());
                if (javaDocStringBuilder.isJavaDocCommentCompleted()) {
                    String javaDocString = javaDocStringBuilder.getJavaDocAndReset();
                    attachJavaDocOnNextDocumentableElement(javaDocString, node);
                }
            }
            else {
                javaDocStringBuilder.reset();
            }
        }
    }

    private void attachJavaDocOnNextDocumentableElement(String javaDocString, INode node) {
        INode nextSibling = node.getNextSibling();
        IDocumentable documentableElement = findIDocumentableInHierarchy(NodeModelUtils.findActualSemanticObjectFor(nextSibling));
        if (documentableElement != null && documentableElement.getTxtDescription() == null) {  
            documentableElement.setTxtDescription(javaDocString);
        }
    }

    private IDocumentable findIDocumentableInHierarchy(EObject semanticElement) {
        if (semanticElement == null)
            return null;
        if (semanticElement instanceof IDocumentable)
            return (IDocumentable) semanticElement;
        return findIDocumentableInHierarchy(semanticElement.eContainer());
    }


}
