package org.openxma.dsl.core.validation;

import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.naming.IQualifiedNameProvider;
import org.eclipse.xtext.naming.QualifiedName;

import com.google.common.base.Function;

public class ValidationCollectionUtil {

	public static EObject findFirstElementWithId(Collection<EObject> collection, final String id,
			Function<EObject, String> function) {
		for (EObject t : collection) {
			String currentId = function.apply(t);
			if (id.equals(currentId))
				return t;
		}
		return null;
	}

	public static <U> U findFirstElementWithId(Collection<EObject> collection, final Class<U> clazz, final String id,
			Function<EObject, String> function) {
		for (EObject t : collection) {
			if (clazz.isInstance(t)) {
				String currentId = function.apply(t);
				if (id.equals(currentId))
					return (U) t;
			}
		}
		return null;
	}

	/**
	 * 
	 * @param collection
	 * @param id
	 * @return
	 */
	public static int countElementsWithId(Collection<EObject> collection, final String id, Function<EObject, String> function) {
		int count = 0;
		for (EObject obj : collection) {
			String currentId = function.apply(obj);
			if (id.equals(currentId))
				count++;
		}
		return count;
	}

	public static <T> int countElementsWithId(Collection<EObject> collection, final Class<T> clazz, final String id,
			Function<EObject, String> function) {
		int count = 0;
		for (EObject obj : collection) {
			if (clazz.isInstance(obj)) {
				String currentId = function.apply(obj);
				if (id.equals(currentId))
					count++;
			}
		}
		return count;
	}

	/**
	 * 
	 * @param collection
	 * @param guiElementId
	 * @return
	 */
	public static boolean hasDuplicateElementsWithId(Collection<EObject> collection, final QualifiedName guiElementId,
			IQualifiedNameProvider qualifiedNameProvider) {
		int count = 0;
		for (EObject obj : collection) {
			QualifiedName currentId = qualifiedNameProvider.apply(obj);
			if (guiElementId.equals(currentId)) {
				count++;
				if (count > 1)
					return true;
			}
		}
		return false;
	}

	/**
	 * 
	 * @param <T>
	 * @param collection
	 * @param clazz
	 * @param id
	 * @return
	 */
	public static <T> boolean hasDuplicateElementsWithId(Collection<EObject> collection, final Class<T> clazz,
			final String id, Function<EObject, String> function) {
		int count = 0;
		for (EObject obj : collection) {
			if (clazz.isInstance(obj)) {
				String currentId = function.apply(obj);
				if (id.equals(currentId)) {
					count++;
					if (count > 1)
						return true;
				}
			}
		}
		return false;
	}

}
